#!/bin/bash

# certtool uses a default key size of 1024 bits which is not allowed by Debian policy
# (you would get some obscure error in this case)
# Medium creates a key of 2048 bits
KEY_SECURITY=Medium

CA_CERT=ca-cert.pem
SERVER_CERT=server-cert.pem
SERVER_KEY=server-key.pem
CA_KEY="key.$RANDOM.$$.tmp"

abort() {
    echo $1
    exit 1
}

# make CA priv/pub key
mk_ca() {
    # Certificate Authority
    [ -f $CA_KEY ] || certtool --generate-privkey --sec-param $KEY_SECURITY > $CA_KEY
    if [ $? -ne 0 ]; then abort "Failed generating CA private key."; fi
    chmod 0400 $CA_KEY    
    if [ $? -ne 0 ]; then abort "Failed changing CA private key permissions."; fi

    [ -f $CA_CERT ] || certtool --generate-self-signed \
               --load-privkey $CA_KEY \
               --template $1 \
               --outfile $CA_CERT

    if [ $? -ne 0 ]; then abort "Failed generating self-signed certificate."; fi
}

# make server priv key and CA sign request
mk_server_cert() {
    [ -f $CA_KEY -a -f $CA_CERT ] || mk_ca $1
    
    # server certificates
    SRV_INFO=$2
    SRV_OUTPUTDIR=$3
    [ -d "$SRV_OUPUTDIR" ] || mkdir $SRV_OUTPUTDIR
    if [ $? -ne 0 ]; then abort "Failed creating output directory."; fi
    
    SRV_KEY="$SRV_OUTPUTDIR/$SERVER_KEY"
    [ -f "$SRV_KEY" ] || certtool --generate-privkey --sec-param $KEY_SECURITY > $SRV_KEY
    if [ $? -ne 0 ]; then abort "Failed generating server private key."; fi
    chmod 0400 $SRV_KEY
    if [ $? -ne 0 ]; then abort "Failed changing server private key permissions."; fi
    
    SRV_CERT="$SRV_OUTPUTDIR/$SERVER_CERT"
    certtool --generate-certificate \
         --load-ca-certificate $CA_CERT \
         --load-ca-privkey $CA_KEY \
         --load-privkey $SRV_KEY \
         --template $SRV_INFO \
         --outfile $SRV_CERT

    if [ $? -ne 0 ]; then abort "Failed generating keys."; fi

    mv $CA_CERT $SRV_OUTPUTDIR
    if [ $? -ne 0 ]; then abort "Failed moving CA certificate to final directory."; fi
    rm -f $CA_KEY
    if [ $? -ne 0 ]; then abort "Failed removing temporary CA key."; fi
}

app=`basename $0`

if [ $# -ne 3 ]; then
    echo "Usage: $app CA_INFO SERVER_INFO OUTPUT_DIR"
    echo "    CA_INFO is the certificate authority info file (e.g. ca.info)"
    echo "    SERVER_INFO is the server certificate info file (e.g. nexus-server.info)"
    echo "    OUTPUT_DIR is a directory (non-existing) where to create the certificates"
    exit 1
fi

[ -f $1 ] || abort "File not found: \"$1\"."
[ -f $2 ] || abort "File not found: \"$2\"."
[ -d $3 ] && abort "Directory already exists: \"$3\"."

mk_server_cert $1 $2 $3
