package cmdVM

import (
	"fmt"
	"sync"
    "errors"
    "encoding/json"
    "nexus-common/vm"
	"nexus-common/params"
	"nexus-client/exec"
    u "nexus-client/utils"
    g "nexus-client/globals"
    "github.com/go-playground/validator/v10"
)

// The boolean indicates whether attaching is blocking (synchronous) or non-blocking (asynchronous).
func AttachToVMs(vms []vm.VMAttachCredentialsSerialized, synchronous bool) (int, error) {
	statusCode := 0

    hostname := g.GetInstance().Hostname
    cert := g.GetInstance().PubCert

    client := g.GetInstance().Client
    host := g.GetInstance().Host

	var wg sync.WaitGroup

	if synchronous {
		// Use wait groups to wait until all viewers threads have completed.
		wg.Add(len(vms))
	}

    for _, v := range(vms) {
        uuid := v.ID.String()
        p := &params.VMAttachCreds{ Pwd: v.Pwd }
        resp, err := client.R().SetBody(p).Post(host+"/vms/"+uuid+"/spicecreds")
        if err != nil {
			return 1, err
        }

        if resp.IsSuccess() {
            var creds vm.VMSpiceCredentialsSerialized
            if err := json.Unmarshal(resp.Body(), &creds); err != nil {
				return 1, err
            }
            if err := validator.New(validator.WithRequiredStructEnabled()).Struct(creds); err != nil {
                return 1, err
            }

            go func(v vm.VMAttachCredentialsSerialized, creds vm.VMSpiceCredentialsSerialized) {
                stdoutStderr, err := exec.RunRemoteViewer(hostname, cert, creds.Name, creds.SpicePort, creds.SpicePwd, false)
                if err != nil {
                    u.PrintlnErr("Failed attaching to VM ", v.ID, ": ", fmt.Sprintf("%s", stdoutStderr))
                    statusCode |= 1
                }
				if synchronous {
					wg.Done()
				}
            } (v, creds)

        } else {
			return 1, errors.New("Error: "+resp.Status()+": "+resp.String())
        }
    }

	if synchronous {
		wg.Wait()
	}

    return statusCode, nil
}
